<?php
/**
 * SkeletonCart
 * (c) 2017-2023 Juxtaposition Inc. All Rights Reserved.
 * (c) 2017-2023 Curlybracket, Inc. All Rights Reserved.
 * This code cannot be redistributed without permission from the copyright holder.
 * For more information, consult your SkeletonCart license.
**/

namespace App;

class Cart extends Error implements \Iterator, \Countable, \ArrayAccess
{
    protected $sameGroupIdOnly = true;
    protected $useStockManagement = false;
    protected $maxWeight = 0;
    protected $itemChecker = array();

    protected $items = array();
    protected $subTotalPrice = 0;
    protected $totalWeight = 0;
    protected $tax = 0;
    protected $payment = 0;
    protected $created = 0;
    protected $revision = 0;

    public function __construct($sameGroupIdOnly = true, $useStockManagement = false, $maxWeight = 0, $revision = 0)
    {
        $this->sameGroupIdOnly = $sameGroupIdOnly;
        $this->useStockManagement = $useStockManagement;
        $this->maxWeight = $maxWeight;
        $this->created = date("YmdHis");
        $this->revision = $revision;
    }

    public function getSameGroupIdOnly()
    {
        return $this->sameGroupIdOnly;
    }

    public function getUseStockManagement()
    {
        return $this->useStockManagement;
    }

    public function getMaxWeight()
    {
        return $this->maxWeight;
    }

    public function getRevision()
    {
        return $this->revision;
    }

    public function load($session)
    {
        $cart = $session->get('cart');
        if ($cart && $cart instanceof Cart) {
            // NOTE: カートの属性が一致しない場合、中身を読み込まない。
            // つまり、init.php の設定で影響のある項目を変更すると、それ以前にカートに入っていた商品は消える。
            if ($this->useStockManagement === $cart->useStockManagement
                && $this->maxWeight === $cart->maxWeight
                && $this->sameGroupIdOnly === $cart->sameGroupIdOnly
                && $this->revision === $cart->revision
            ) {
                try {
                    $this->items = $cart->items;
                    $this->updateTotal();
                } catch (\Exception $e) {
                    // NOTE: load時にエラーが発生した場合は、以降の操作が不可になるのでカートを空にする。
                    $session->set('cart', null);
                    $session->save();
                    throw $e;
                }
            } else {
                \App::logger()->debug('カート属性が異なるため読み込めません。');
            }
        }
    }

    public function updateTotal()
    {
        $price = 0;
        $weight = 0;
        foreach ($this->items as $item) {
            $price += $item->totalPrice();
            $weight += $item->totalWeight();
        }
        $this->subTotalPrice = $price;
        $this->totalWeight = $weight;

        $config = \App::config();
        $this->tax = $this->subTotalPrice * $config->getTaxRate();
        $this->payment = $this->subTotalPrice + $this->tax;

        \App::runCallbacks('updateCart', $this);
    }

    public function save($session)
    {
        $this->updateTotal();
        $session->set('cart', $this);
        $session->save();
    }

    public function findItem($item)
    {
        if (!isset($item)) {
            throw new \UnexpectedValueException('Missing argument');
        }

        foreach ($this->items as $_item) {
            if ($_item->id() === $item->id()
                && $_item->options() == $item->options()
            ) {
                return $_item;
            }
        }

        return null;
    }

    public function addItem($item, $check = false)
    {
        if (!isset($item)) {
            throw new \UnexpectedValueException('Missing argument');
        }

        if(!$this->isSameGroupId($item)) {
            throw new GroupIdMismatchException('異なるグループIDの商品は同時に購入できません。');
        }

        $_item = $this->findItem($item);
        if($_item) {
            // 個数をマージする。
            $item->addAmount($_item->amount());
        }

        if($check) {
            if($this->checkItem($item)) {
                throw new \App\InvalidCartException($item->error());
            }
            if($this->checkMaxWeightForItem($item)) {
                throw new \App\MaxWeightExceededException($item->error());
            }
        }

        if($_item) {
            // マージ済みの個数をセットする。
            $_item->setAmount($item->amount());
        }
        else {
            array_push($this->items, $item);
        }

        $this->updateTotal();
        return true;
    }

    public function sameGroupIdOnly()
    {
        return $this->sameGroupIdOnly;
    }

    public function removeItem($itemId)
    {
        if (!isset($itemId)) {
            throw new \UnexpectedValueException('Missing argument');
        }
        $idx = -1;
        foreach ($this->items as $index => $item) {
            if ($item->id() === $itemId) {
                $idx = $index;
                break;
            }
        }
        $items = null;
        if ($idx >= 0) {
            $items = array_splice($this->items, $idx, 1);
            $this->updateTotal();
        }
        return isset($items[0]) ? $items[0] : null;
    }

    public function getItem($itemId)
    {
        if (!isset($itemId)) {
            throw new \UnexpectedValueException('Missing argument');
        }
        foreach ($this->items as $item) {
            if ($item->id() === $itemId) {
                return $item;
            }
        }
        return null;
    }

    public function items()
    {
        return $this->items;
    }

    public function subTotalPrice()
    {
        return $this->subTotalPrice;
    }

    public function setSubTotalPrice($subTotalPrice)
    {
        $this->subTotalPrice = $subTotalPrice;
    }

    public function totalWeight()
    {
        return $this->totalWeight;
    }

    public function setTotalWeight($totalWeight)
    {
        $this->subTotalPrice = $totalWeight;
    }

    public function tax()
    {
        return $this->tax;
    }

    public function setTax($tax)
    {
        $this->tax = $tax;
    }

    public function payment()
    {
        return $this->payment;
    }

    public function setPayment($payment)
    {
        $this->payment = $payment;
    }

    public function addItemChecker(CartItemChecker $checker)
    {
        $this->itemChecker[] = $checker;
    }

    public function isValid()
    {
        $this->clearError();

        $this->checkItems();
        $this->checkMaxWeight();

        $hasItemError = array_reduce($this->items, function ($carry, $item) {
            return $item->hasError() ? $carry + 1 : $carry;
        }, 0);
        if ($hasItemError > 0) {
            if (!$this->hasError()) {
                $this->setError('カート内に注文できない商品があります。');
            }
        }
        if ($this->hasError()) {
            return false;
        }
        return true;
    }

    public function isSameGroupId($item)
    {
        foreach ($this->items as $_item) {
            if ($this->sameGroupIdOnly() && $_item->groupId() !== $item->groupId()) {
                return false;
            }
        }
        return true;
    }

    public function checkItem($item)
    {
        $item->clearError();

        foreach ($this->itemChecker as $checker) {
            $checker->check($item);
        }

        return $item->hasError();
    }

    public function checkItems()
    {
        foreach ($this->itemChecker as $checker) {
            foreach ($this->items as $item) {
                $checker->check($item);
            }
        }
    }

    public function checkMaxWeightForItem($item)
    {
        $item->clearError();

        if($this->maxWeight > 0) {
            if(($this->totalWeight + $item->totalWeight()) > $this->maxWeight) {
                $item->setError('重量制限を超えています。');
            }
        }

        return $item->hasError();
    }

    public function checkMaxWeight()
    {
        if ($this->maxWeight > 0) {
            if ($this->totalWeight > $this->maxWeight) {
                $this->setError("重量制限を超えています。");
            }
        }
    }

    public function allocateStock(StockAllocator $stockAllocator)
    {
        if (!isset($stockAllocator)) {
            throw new \UnexpectedValueException('Missing argument - stockAllocator');
        }
        $stockAllocator->allocate();
    }

    public function getCompatibleArray()
    {
        $items = array_map(function ($item) {
            return $item->getCompatibleProperties();
        }, $this->items);
        return $items;
    }

    public function getItemsArray()
    {
        $items = array_map(function ($item) {
            return $item->getProperties();
        }, $this->items);
        return $items;
    }

    public function totalAmount()
    {
        $totalAmount = 0;
        foreach ($this->items as $item) {
            $totalAmount += $item->amount();
        }
        return $totalAmount;
    }

    public function isEmpty()
    {
        return $this->count() == 0;
    }

    public function count(): int
    {
        return count($this->items);
    }

    public function current(): mixed
    {
        return current($this->items);
    }

    // Iterator interface
    #[\ReturnTypeWillChange]
    public function key()
    {
        return key($this->items);
    }

    public function next(): void
    {
        next($this->items);
    }

    public function rewind(): void
    {
        reset($this->items);
    }

    public function valid(): bool
    {
        return key($this->items) !== null;
    }

    public function offsetExists($offset): bool
    {
        return isset($this->items[$offset]);
    }

    #[\ReturnTypeWillChange]
    public function offsetGet($offset)
    {
        return isset($this->items[$offset]) ? $this->items[$offset] : null;
    }

    // ArrayAccess interface
    public function offsetSet($offset, $value): void
    {
        $this->updateItemByIndex($offset, $value);
    }

    public function updateItemByIndex($index, $amount, $check = false)
    {
        if (!isset($index)) {
            throw new \UnexpectedValueException('Missing argument - index');
        }
        if ($index < 0 || $index >= count($this->items)) {
            throw new \OutOfRangeException('指定された商品が見つかりません。');
        }
        if (!isset($amount) || Util::is_empty($amount) || !is_numeric($amount)) {
            throw new ValidationException('パラメータの値が不正です。', array('amount' => '数量には値を指定してください。'));
        }
        if ($amount == 0) {
            return $this->removeItemByIndex($index);
        }
        $item = $this->items[$index];
        $item->setAmount($amount);

        if($check) {
            if($this->checkItem($item)) {
                throw new \App\InvalidCartException($item->error());
            }
            if($this->checkMaxWeightForItem($item)) {
                throw new \App\MaxWeightExceededException($item->error());
            }
        }

        $this->updateTotal();
        return true;
    }

    public function removeItemByIndex($index)
    {
        if (!isset($index)) {
            throw new \UnexpectedValueException('Missing argument');
        }
        if ($index < 0 || $index >= count($this->items)) {
            throw new \OutOfRangeException('指定された商品が見つかりません。');
        }
        $items = array_splice($this->items, $index, 1);
        $this->updateTotal();
        return isset($items[0]) ? $items[0] : null;
    }

    public function offsetUnset($offset): void
    {
        $this->removeItemByIndex($offset);
    }
}

