<?php
/**
 * SkeletonCart
 * (c) 2017-2023 Juxtaposition Inc. All Rights Reserved.
 * (c) 2017-2023 Curlybracket, Inc. All Rights Reserved.
 * This code cannot be redistributed without permission from the copyright holder.
 * For more information, consult your SkeletonCart license.
 **/

namespace App;

use \App\Util;

class CartController extends Controller
{
    public function __construct($type = null, $mode = null)
    {
        if (is_null($type)) {
            $type = 'cart';
        }
        parent::__construct($type, $mode);
    }

    public function _process()
    {
        try {
            $command = $this->getCommand();
            switch ($command) {
                case 'v':
                    $this->show();
                    break;
                case 'ad':
                    $this->addItem();
                    break;
                case 'dl':
                    $this->removeItem();
                    break;
                case 'ud':
                    $this->updateItem();
                    break;
                default:
                    $this->show();
                    break;
            }
        } catch (InvalidCartException $e) {
            \App::logger()->error(__METHOD__ . ": " . get_class($e) . ": " . $e->getMessage());
            $this->redirect_to_cart();
        } catch (\Exception $e) {
            $data = $this->getTemplateParams();
            $data = $this->addErrorMessage($data, $e);
            \App::logger()->error(__METHOD__ . ": " . get_class($e) . ": " . $e->getMessage());
            $this->render('_error.html', $data);
        }
    }

    public function process()
    {
        try {
            switch ($this->mode) {
                case 'show':
                    $this->show();
                    break;
                case 'add':
                    $this->addItem();
                    break;
                case 'remove':
                    $this->removeItem();
                    break;
                case 'update':
                    $this->updateItem();
                    break;
                default:
                    $this->show();
                    break;
            }
        } catch (InvalidCartException $e) {
            \App::logger()->error(__METHOD__ . ": " . get_class($e) . ": " . $e->getMessage());
            \App::logger()->debug(__METHOD__ . ": " . $e->getTraceAsString());
            $this->redirect_to_cart();
        } catch (\Exception $e) {
            \App::logger()->error(__METHOD__ . ": " . get_class($e) . ": " . $e->getMessage());
            \App::logger()->debug(__METHOD__ . ": " . $e->getTraceAsString());
            $data = $this->getTemplateParams();
            $data = $this->addErrorMessage($data, $e);
            $this->render('_error.html', $data);
        }
    }

    public function getCompatibleParams()
    {
        $request = \App::request();
        $params = array(
            'id' => $request->data->cart_item_item_id,
            'amount' => Util::zen2han($request->data->cart_item_amount),
        );
        $options = array();
        foreach (range(1, 30) as $n) {
            if (isset($request->data['cart_item_opt' . $n])) {
                $options[$n] = array(
                    'value' => $request->data['cart_item_opt' . $n],
                );
            }
        }
        if (count($options) > 0) {
            $params['options'] = $options;
        }

        $item_num = $this->getItemIndex();
        if (isset($item_num)) {
            $params['index'] = $this->_num2index($item_num);
        }
        if (!isset($params['index']) && isset($request->data['cart_item_num'])) {
            $params['index'] = $this->_num2index($request->data['cart_item_num']);
        }

        return $params;
    }

    public function getAddParams()
    {
        $request = \App::request();
        $params = array(
            'id' => $request->data->id,
            'amount' => Util::zen2han($request->data->amount),
        );
        $options = array();
        foreach (range(1, 30) as $n) {
            if (isset($request->data['opt' . $n])) {
                $options[$n] = array(
                    'value' => $request->data['opt' . $n],
                );
            }
        }
        if (count($options) > 0) {
            $params['options'] = $options;
        }
        return $params;
    }

    public function getIndex()
    {
        $request = \App::request();
        $index = $request->data->index;
        if (is_numeric($index)) {
            $index = intval($index);
        } else {
            $index = 0;
        }
        return $index;
    }

    public function getRemoveParams()
    {
        $params = array(
            'index' => $this->getIndex(),
        );
        return $params;
    }

    public function getAmount($index)
    {
        $request = \App::request();
        $amount = $request->data->amount;
        if (is_array($amount) && isset($amount[$index])) {
            $amount = $amount[$index];
        }
        return intval(Util::zen2han($amount));
    }

    public function getUpdateParams()
    {
        $request = \App::request();
        $index = $this->getIndex();
        $params = array(
            'index' => $index,
            'amount' => $this->getAmount($index),
        );
        return $params;
    }

    private function _num2index($n)
    {
        return $n > 0 ? --$n : 0;
    }

    public function show()
    {
        $service = \App::cartService();
        $service->checkCart();
        $data = $this->getTemplateParams();
        $this->render('_cart.html', $data);
    }

    public function addItem()
    {
        if (\App::request()->method !== 'POST') {
            throw new \RuntimeException('Invalid Method');
        }
        $params = $this->getAddParams();
        $service = \App::cartService();
        $service->addItem($params);
        $this->redirect_to_cart();
    }

    public function removeItem()
    {
        if (\App::request()->method !== 'POST') {
            throw new \RuntimeException('Invalid Method');
        }
        $params = $this->getRemoveParams();
        $service = \App::cartService();
        $service->removeItem($params);
        $this->redirect_to_cart();
    }

    public function updateItem()
    {
        if (\App::request()->method !== 'POST') {
            throw new \RuntimeException('Invalid Method');
        }
        $params = $this->getUpdateParams();
        $service = \App::cartService();
        try {
            $service->updateItem($params);
        } catch (ValidationException $e) {
            $errors = $e->getErrors();
        }
        if (isset($errors)) {
            $data = $this->getTemplateParams();
            $data = $this->addValidationErrorMessage($data, $errors);
            $this->render('_cart.html', $data);
        } else {
            $this->redirect_to_cart();
        }
    }

}

