<?php
/**
 * SkeletonCart
 * (c) 2017-2023 Juxtaposition Inc. All Rights Reserved.
 * (c) 2017-2023 Curlybracket, Inc. All Rights Reserved.
 * This code cannot be redistributed without permission from the copyright holder.
 * For more information, consult your SkeletonCart license.
 **/

namespace App;

class CartItem extends Error
{
    protected $id;
    protected $name;
    protected $price = 0;     // itemからコピーした価格を保持する (read/write)
    protected $unitPrice = 0; // itemからコピーした価格を保持する (readonly)
    protected $groupId;
    protected $weight = 0;
    protected $amount = 0;
    protected $totalPrice = 0;
    protected $totalWeight = 0;
    protected $options = array();

    public function __construct($params = null)
    {
        $this->setProperties($params);
    }

    public function setProperties($params)
    {
        if (!isset($params)) {
            return;
        }

        $this->validateParams($params);

        $this->id = $params['id'];
        $this->name = $params['name'];
        $this->price = $params['price'];
        $this->unitPrice = $params['price'];
        $this->groupId = $params['groupId'];
        $this->weight = $params['weight'];
        $this->amount = $params['amount'];
        if (isset($params['options'])) {
            $this->options = $params['options'];
        }

        $this->update();
    }

    protected function validateParams($params)
    {
        $rules = array(
            'id' => array(
                'REQUIRED' => '商品IDを指定してください。',
            ),
            'name' => array(
                'REQUIRED' => '商品名を指定してください。',
            ),
            'price' => array(
                'REQUIRED' => '価格を指定してください。',
                'NUMERIC' => '価格は数値を指定してください。',
            ),
            'groupId' => array(
                'REQUIRED' => 'グループIDを指定してください。',
            ),
            'weight' => array(
                'REQUIRED' => '重量を指定してください。',
                'NUMERIC' => '重量は数値を指定してください。',
            ),
            'amount' => array(
                'REQUIRED' => '数量を指定してください。',
                'NUMERIC' => '数量は数値を指定してください。',
            ),
            'options' => array(
                'CUSTOM' => array(
                    'func' => array($this, 'validateOptions'),
                    'message' => 'オプションの最大個数を超えています。',
                ),
                'CUSTOM_OPT' => array(
                    'func' => array($this, 'validateOption'),
                    'message' => '不正なオプションの形式です。',
                ),
            ),
        );
        $validator = new Validator($rules);
        if (!$validator->validate($params)) {
            $errors = $validator->getErrors();
            throw new ValidationException('パラメータの値が不正です。', $errors);
        }
    }

    public function update()
    {
        $this->totalPrice = $this->price * $this->amount;
        $this->totalWeight = $this->weight * $this->amount;

        \App::runCallbacks('updateCartItem', $this);
    }

    public function validateOptions($key, $value, $params, $args)
    {
        if (isset($value)
            && is_array($value)
            && count($value) > \App::config()->maxNumOptions()
        ) {
            return false;
        }
        return true;
    }

    public function validationOption($key, $value, $params, $args)
    {
        if (!is_array($value)) {
            return false;
        }
        foreach ($value as $opt_k => $opt_v) {
            if (!is_array($opt_v)) {
                return false;
            }
            if (!isset($opt_v['value'])) {
                return false;
            }
        }
        return true;
    }

    // Accessors

    public function id()
    {
        return $this->id;
    }

    public function name()
    {
        return $this->name;
    }

    public function price()
    {
        return $this->price;
    }

    public function setPrice($price)
    {
        $this->price = $price;
        // HACK: updateCartItem コールバックから呼び出されると無限ループになるので update() を呼び出さない。
    }

    public function unitPrice()
    {
        return $this->unitPrice;
    }

    public function groupId()
    {
        return $this->groupId;
    }

    public function weight()
    {
        return $this->weight;
    }

    public function amount()
    {
        return $this->amount;
    }

    public function setAmount($amount)
    {
        $this->amount = $amount;
        $this->update();
    }

    public function addAmount($amount)
    {
        $this->amount += $amount;
        $this->update();
    }

    public function options()
    {
        return $this->options;
    }

    public function setOption($n, $value)
    {
        if ($n < 1 || $n > \App::config()->maxNumOptions()) {
            throw new \UnexpectedValueException('Invalid option number');
        }
        $this->options[$n] = $value;
    }

    public function totalPrice()
    {
        return $this->totalPrice;
    }

    public function setTotalPrice($totalPrice)
    {
        $this->totalPrice = $totalPrice;
    }

    public function totalWeight()
    {
        return $this->totalWeight;
    }

    public function setTotalWeight($totalWeight)
    {
        return $this->totalWeight = $totalWeight;
    }

    // Compatible Accessors
    public function cart_item_item_id()
    {
        return $this->id;
    }

    public function cart_item_name()
    {
        return $this->name;
    }

    public function cart_item_price()
    {
        return $this->price;
    }

    public function cart_item_group_id()
    {
        return $this->groupId;
    }

    public function cart_item_weight()
    {
        return $this->weight;
    }

    public function cart_item_amount()
    {
        return $this->amount;
    }

    public function getCompatibleProperties()
    {
        $props = array(
            'cart_item_item_id' => $this->id,
            'cart_item_name' => $this->name,
            'cart_item_price' => $this->price,
            'cart_item_group_id' => $this->groupId,
            'cart_item_weight' => $this->weight,
            'cart_item_amount' => $this->amount,
            'cart_item_price_sum' => $this->totalPrice,
            'cart_item_weight_sum' => $this->totalWeight,
            'cart_item_error' => $this->error(),
        );
        foreach (range(1, \App::config()->maxNumOptions()) as $n) {
            if (isset($this->options[$n])) {
                if (isset($this->options[$n]['value'])) {
                    $props["cart_item_opt{$n}"] = $this->options[$n]['value'];
                } else {
                    $props["cart_item_opt{$n}"] = $this->options[$n]['value'];
                }
                foreach ($this->options[$n] as $k => $v) {
                    $props["cart_item_opt{$n}_{$k}"] = $v;
                    $props["cart_item_{$k}_opt{$n}"] = $v;
                }
            } else {
                $props["cart_item_opt{$n}"] = null;
            }
        }
        return $props;
    }

    public function getProperties()
    {
        $props = array(
            'id' => $this->id,
            'name' => $this->name,
            'price' => $this->price,
            'group_id' => $this->groupId,
            'weight' => $this->weight,
            'amount' => $this->amount,
            'price_sum' => $this->totalPrice,
            'weight_sum' => $this->totalWeight,
            'error' => $this->error(),
        );
        foreach (range(1, \App::config()->maxNumOptions()) as $n) {
            if (isset($this->options[$n])) {
                if (isset($this->options[$n]['value'])) {
                    $props["opt{$n}"] = $this->options[$n]['value'];
                } else {
                    $props["opt{$n}"] = $this->options[$n]['value'];
                }
                foreach ($this->options[$n] as $k => $v) {
                    $props["opt{$n}_{$k}"] = $v;
                    $props["{$k}_opt{$n}"] = $v;
                }
            } else {
                $props["opt{$n}"] = null;
            }
        }
        return $props;
    }
}
