<?php
/**
 * SkeletonCart
 * (c) 2017-2023 Juxtaposition Inc. All Rights Reserved.
 * (c) 2017-2023 Curlybracket, Inc. All Rights Reserved.
 * This code cannot be redistributed without permission from the copyright holder.
 * For more information, consult your SkeletonCart license.
 **/

namespace App;

class Controller
{
    protected $type;
    protected $mode;

    public function __construct($type = null, $mode = null)
    {
        $this->type = $type;
        $this->mode = $mode;
    }

    public function getControllerName()
    {
        return \App::request()->query->qp0;
    }

    public function getCommand()
    {
        return \App::request()->query->qp1;
    }

    public function getItemIndex()
    {
        return \App::request()->query->qp2;
    }

    public function getMode()
    {
        return $this->mode;
    }

    public function setMode($mode)
    {
        $this->mode = $mode;
    }

    public function getType()
    {
        return $this->type;
    }

    public function prepare()
    {
        $config = \App::config();
        $maintenanceMode = $config->getMaintenanceMode();
        if ($maintenanceMode) {
            $now = time();
            $start = $config->getMaintenanceTimeStart();
            $end = $config->getMaintenanceTimeEnd();
            if ($now < $start || $now > $end) {
                return;
            }
            $data = array(
                'maintenance_start' => $start,
                'maintenance_end' => $end,
                'app_script' => \App::appScriptName(),
                'app_dir' => \App::appScriptDir(),
                'base_url' => \App::appBasedir(),
            );
            $this->render('_maintenance.html', $data, 503);
        }
    }

    public function start($mode = null)
    {
        if (!is_null($mode)) {
            $this->setMode($mode);
        }
        try {
            \App::runCallbacks('preStart', $this);
            $this->prepare();
            $this->process();
            \App::runCallbacks('postStart', $this);
        }
        catch (\Exception $e) {
            \App::logger()->error(__METHOD__ . ": " . get_class($e) . ": " . $e->getMessage());
            \App::logger()->debug(__METHOD__ . ": " . $e->getTraceAsString());
            $data = $this->getTemplateParams();
            $data = $this->addErrorMessage($data, $e);
            $this->render('_error.html', $data);
        }
    }

    public function process()
    {
    }

    public function json($data, $code = 200, $encode = true, $charset = 'utf-8', $option = 0)
    {
        if (!headers_sent()) {
            $http_origin = isset($_SERVER['HTTP_ORIGIN']) ? $_SERVER['HTTP_ORIGIN'] : null;
            $origin = \App::config()->getAllowedOrigin();
            if(isset($http_origin) && isset($origin) && $http_origin === $origin) {
                header("Access-Control-Allow-Origin: $origin");
                header("Access-Control-Allow-Credentials: true");
            }
        }
        \App::json($data, $code, $encode, $charset, $option);
    }

    public function render($template, $data, $code = 200)
    {
        if(\App::isAcceptJson()) {
            $this->json($data, $code);
        }
        else {
            \App::render($template, $data);
            if($code != 200) {
                \App::stop($code);
            }
        }
    }

    public function redirect_to_cart()
    {
        $url = \App::config()->getCartPath();
        $this->redirect($url);
    }

    public function redirect($url, $code = 303)
    {
        \App::redirect($url, $code);
    }

    public function base_url()
    {
        $base_url = \App::appBasedir();
        return $base_url == '/' ? '' : $base_url;
    }

    public function getTemplateParams()
    {
        $cart = \App::cart();
        $config = \App::config();
        $data = array(
            'error_id' => null,
            'error_message' => null,
            'items' => $cart->getItemsArray(),
            'subtotal' => $cart->subTotalPrice(),
            'tax' => $cart->tax(),
            'total_weight' => $cart->totalWeight(),
            'weight' => $cart->totalWeight(),
            'free_delivery_price' => $config->getFreePostagePrice(),
            'payment' => $cart->payment(),
            'tax_rate' => $config->getTaxRate(),
            'app_script' => \App::appScriptName(),
            'app_dir' => \App::appScriptDir(),
            'base_url' => $this->base_url(),
        );
        $labels = $config->cartItemOptionLabels();
        $data = array_merge($data, $labels);

        if ($cart->hasError()) {
            $data = $this->addCartError($data, $cart);
        }
        return $data;
    }

    public function addCartError($data, $cart)
    {
        $errors = array();
        if ($cart->hasError()) {
            $errors[] = $cart->error();
            foreach ($cart as $cartItem) {
                if ($cartItem->hasError()) {
                    $errors[] = $cartItem->error();
                }
            }
        }
        if (!empty($errors)) {
            $data['error_id'] = 'cart_error';
            $data['error_message'] = implode("\n", $errors);
        }
        return $data;
    }

    public function addErrorMessage($data, $e)
    {
        if ($e instanceof MaxWeightExceededException) {
            $data = $this->addErrorInfo($data, 'weight', $e->getMessage());
        } elseif ($e instanceof OutOfStockException) {
            $data = $this->addErrorInfo($data, 'soldout', $e->getMessage());
        } elseif ($e instanceof GroupIdMismatchException) {
            $data = $this->addErrorInfo($data, 'different_group_id', $e->getMessage());
        } elseif ($e instanceof ItemsFileException) {
            $data = $this->addErrorInfo($data, 'illegal_item_dat', $e->getMessage());
        } elseif ($e instanceof IllegalItemDataException) {
            $data = $this->addErrorInfo($data, 'illegal_item_dat', $e->getMessage());
        } else {
            $data = $this->addErrorInfo($data, null, $e->getMessage());
        }
        return $data;
    }

    public function addErrorInfo($data, $id, $message)
    {
        $error = array(
            'error_id' => $id,
            'error_message' => $message,
        );
        return array_merge($data, $error);
    }

    public function addValidationErrorMessage($params, $errors)
    {
        if (!is_array($errors)) {
            return $params;
        }
        $errorMessages = '';
        foreach ($errors as $k => $v) {
            $errorMessages .= $v . "\n";
        }
        $params['error_message'] = $errorMessages;
        return $params;
    }

    public function addInputParams($params)
    {
        $request = \App::request();
        foreach ($request->data as $k => $v) {
            $params[$k] = $v;
        }
        return $params;
    }
}

