<?php
/**
 * SkeletonCart
 * (c) 2017-2023 Juxtaposition Inc. All Rights Reserved.
 * (c) 2017-2023 Curlybracket, Inc. All Rights Reserved.
 * This code cannot be redistributed without permission from the copyright holder.
 * For more information, consult your SkeletonCart license.
 **/

namespace App;

use App\Item;
use App\ItemsFileException;

class FileItemList implements \App\ItemList
{
    protected $fileName;
    protected $map = array();
    protected $sp = '|';
    protected $fp = false;
    protected $loaded = false;
    protected $locked = false;
    protected $fmode = "r";

    public function __construct($fileName)
    {
        $this->fileName = $fileName;
        if (\App::config()->useStockManagement()) {
            $this->fmode = "r+";
        }
        $this->validateFile();
        $this->fp = fopen($this->fileName, $this->fmode);
        if ($this->fp === false) {
            throw new ItemsFileException('Can\'t open items file.');
        }
        $this->lock();
    }

    public function __destruct()
    {
        $this->close();
    }

    public function close()
    {
        if ($this->fp !== false) {
            $this->unlock();
            if (!fclose($this->fp)) {
                throw new ItemsFileException('Can\'t close items file.');
            }
            $this->fp = false;
        }
        return true;
    }

    protected function validateFile()
    {
        if (!is_file($this->fileName)) {
            throw new ItemsFileException('Items file not found.');
        } else if (!is_readable($this->fileName)) {
            throw new ItemsFileException('Can\'t read items file.');
        } else if ($this->fmode === "r+" && !is_writable($this->fileName)) {
            throw new ItemsFileException('Can\'t write items file.');
        }
    }

    protected function fileGetContents()
    {
        if ($this->fp === false) {
            throw new ItemsFileException('Items file is not open.');
        }
        if (fseek($this->fp, 0) === -1) {
            throw new ItemsFileException('Failed to seek to top.');
        }
        $lines = array();
        while (!feof($this->fp)) {
            $lines[] = fgets($this->fp, 16384);
        }
        return $lines;
    }

    protected function filePutContents($content)
    {
        if ($this->fp === false) {
            throw new ItemsFileException('Items file is not open.');
        }
        if (fseek($this->fp, 0) === -1) {
            throw new ItemsFileException('Failed to seek to top.');
        }
        if (ftruncate($this->fp, 0) === false) {
            throw new ItemsFileException('Failed to truncate.');
        }
        $result = fputs($this->fp, $content);
        if ($result !== false) {
            $result = fflush($this->fp);
        }
        if ($result === false) {
            throw new ItemsFileException('Failed to write to items file.');
        }
        return $result;
    }

    public function load()
    {
        if ($this->loaded) {
            return;
        }

        $lines = $this->fileGetContents();
        foreach ($lines as $value) {
            $line = trim($value);
            if (empty($line)) {
                continue;
            }
            $columns = explode($this->sp, $line);
            $this->map[$columns[0]] = $columns;
        }
        $this->loaded = true;
        return true;
    }

    protected function columns2str($columns)
    {
        $columns = array_map(function ($column) {
            return mb_ereg_replace("\|", "", $column);
        }, $columns);
        $str = implode('|', $columns);
        return $str;
    }

    public function save()
    {
        if ($this->fmode === "r") {
            return true;
        }

        $content = '';
        foreach ($this->map as $id => $columns) {
            $line = $this->columns2str($columns);
            $content .= $line . "\n";
        }

        $backupFileName = $this->fileName . ".bck";
        if (is_file($backupFileName)) {
            unlink($backupFileName);
        }
        copy($this->fileName, $backupFileName);

        $result = $this->filePutContents($content);
        return $result;
    }

    public function findOne($id)
    {
        if (!$this->loaded) {
            $this->load();
        }
        if (array_key_exists($id, $this->map)) {
            return $this->map[$id];
        }
        return false;
    }

    public function count()
    {
        return count($this->map);
    }

    protected function props2columns($props)
    {
        $columns = array(
            $props['id'],
            $props['name'],
            $props['price'],
            $props['groupId'],
            $props['stock'],
            $props['weight'],
        );
        return $columns;
    }

    public function update($props)
    {
        $id = $props['id'];
        if (array_key_exists($id, $this->map)) {
            $this->map[$id] = $this->props2columns($props);
            return true;
        }
        return false;
    }

    public function lock()
    {
        if ($this->locked === false) {
            if (!flock($this->fp, LOCK_EX)) {
                throw new ItemsFileException('Can\'t lock items file.');
            }
            $this->locked = true;
        }
        return true;
    }

    public function unlock()
    {
        if ($this->locked === true) {
            if (!flock($this->fp, LOCK_UN)) {
                throw new ItemsFileException('Can\'t unlock items file.');
            }
            $this->locked = false;
        }
        return true;
    }
}

