<?php
/**
 * SkeletonCart
 * (c) 2017-2023 Juxtaposition Inc. All Rights Reserved.
 * (c) 2017-2023 Curlybracket, Inc. All Rights Reserved.
 * This code cannot be redistributed without permission from the copyright holder.
 * For more information, consult your SkeletonCart license.
 **/

namespace App;

use PharIo\Manifest\Email as ManifestEmail;
use StripeSimple\Mailer as StripeSimpleMailer;
use Symfony\Component\Mailer\Transport;
use Symfony\Component\Mailer\Mailer as SymfonyMailer;
use Symfony\Component\Mime\Email;
use Symfony\Component\Mime\Address;

class Mailer
{
    protected $config;
    protected $cart;
    protected $slip;
    protected $view;
    protected $mailer;
    protected $charset;

    public function __construct($config = null, $cart = null, $slip = null, $view = null)
    {
        $this->config = $config;
        $this->cart = $cart;
        $this->slip = $slip;
        $this->view = $view;
        $this->initMailer();
    }

    protected function _initMailer()
    {
        $this->charset = \App::config()->get('mail_charset', 'iso-2022-jp');

        if($this->charset === 'iso-2022-jp') {
            \Swift::init(function () {
                \Swift_DependencyContainer::getInstance()
                                          ->register('mime.qpheaderencoder')
                                          ->asAliasOf('mime.base64headerencoder');
                \Swift_Preferences::getInstance()->setCharset($this->charset);
            });
        }

        $smtp_server = \App::config()->get('smtp_server', array());
        $sendmailCommand = \App::config()->get('sendmail_command');

        $tp = null;
        if(!empty($smtp_server) && $smtp_server['host'] && $smtp_server['port']) {
            $tp = new \Swift_SmtpTransport(
                $smtp_server['host'],
                $smtp_server['port'],
                $smtp_server['encryption'] ?: null
            );
            if($smtp_server['username'] && $smtp_server['password']) {
                $tp->setUsername($smtp_server['username']);
                $tp->setPassword($smtp_server['password']);
            }
        }
        else if($sendmailCommand) {
            $tp = new \Swift_SendmailTransport($sendmailCommand);
        }
        else {
            $tp = new \Swift_SendmailTransport();
        }
        $this->mailer = new \Swift_Mailer($tp);
    }

    public function initMailer()
    {
        $smtp_server = \App::config()->get('smtp_server', array());
        $sendmailCommand = \App::config()->get('sendmail_command');

        $transport = null;
        if(!empty($smtp_server) && $smtp_server['host'] && $smtp_server['port']) {
            $transport = Transport::fromDsn("smtp://{$smtp_server['host']}:{$smtp_server['port']}");
        }
        else if($sendmailCommand) {
            [$cmd, $opts] = explode(' ', $sendmailCommand, 2);
            $opts = rawurlencode(" {$opts}");
            $transport = Transport::fromDsn("sendmail://default?command={$cmd}{$opts}");
        }
        else {
            $transport = Transport::fromDsn('sendmail://default');
        }

        $this->mailer = new SymfonyMailer($transport);
    }

    public function config()
    {
        return $this->config;
    }

    public function setConfig($config)
    {
        $this->config = $config;
    }

    public function cart()
    {
        return $this->cart;
    }

    public function setCart($cart)
    {
        $this->cart = $cart;
    }

    public function slip()
    {
        return $this->slip;
    }

    public function setSlip($slip)
    {
        $this->slip = $slip;
    }

    public function view()
    {
        return $this->view;
    }

    public function setView($view)
    {
        $this->view = $view;
    }

    public function send()
    {
        $this->checkProperties();

        $mailSettings = $this->config->getMailSettings();
        foreach ($mailSettings as $settings) {
            $message = $this->createMessageWithSettings($settings);
            $this->sendMessage($message);
        }
    }

    protected function checkProperties()
    {
        if (!isset($this->config)) {
            throw new \LogicException('Config must be set.');
        }
        if (!isset($this->cart)) {
            throw new \LogicException('Cart must be set.');
        }
        if (!isset($this->slip)) {
            throw new \LogicException('Slip must be set.');
        }
        if (!isset($this->view)) {
            throw new \LogicException('View must be set.');
        }
    }

    protected function createMessageWithSettings($settings)
    {
        if (!isset($settings)) {
            throw new \UnexpectedValueException('Missing argument - settings');
        }

        if (!isset($settings['from'])) {
            throw new \UnexpectedValueException('Missing argument - from');
        }
        $from = $this->createMailAddressArray($settings['from']);

        $to = isset($settings['to']) ? $settings['to'] : null;
        if (empty($to) && isset($settings['toColumn'])) {
            $to = $this->slip->get($settings['toColumn']);
        }
        if (empty($to)) {
            throw new \UnexpectedValueException('Missing argument - To');
        }
        $to = $this->createMailAddressArray($to);

        $replyTo = isset($settings['replyTo']) ? $settings['replyTo'] : null;
        if (isset($settings['replyToUser']) && isset($settings['replyToColumn'])) {
            $replyTo = $this->slip->get($settings['replyToColumn']);
        }
        $replyTo = $this->createMailAddressArray($replyTo);

        /*
        if (!isset($settings['subject'])) {
            throw new \UnexpectedValueException('Missing argument - subject');
        }
        $subject = $settings['subject'];
        if (isset($settings['includeOrderId']) && $settings['includeOrderId']) {
            $subject = sprintf($subject, $this->slip->get('id'));
        }
        */

        $data = $this->getTemplateParams();
        $subject = $this->getSubject($settings, $data);
        $body = $this->view->fetch($settings['template'], $data);
        $this->logOrder("Order Info: \n" . $body);

        $message = $this->createMessage($from, $to, null, null, $replyTo, $subject, $body);
        return $message;
    }

    protected function getSubject($settings, $params)
    {
        if (!isset($settings['subject'])) {
            throw new \UnexpectedValueException('Missing argument - subject');
        }
        $subject = $settings['subject'];
        if (isset($settings['includeOrderId']) && $settings['includeOrderId']) {
            $subject = sprintf($subject, $this->slip->get('id'));
        }
        return $subject;
    }

    protected function createMailAddressArray($string)
    {
        $results = array();
        if(empty($string)) {
            return $results;
        }
        $addrs = explode(',', $string);
        foreach ($addrs as $addr) {
            if (preg_match('/^([^<]*)<([^>]+)>\s*$/', $addr, $matches)) {
                $results[trim($matches[2])] = trim($matches[1]);
            } else {
                $results[] = trim($addr);
            }
        }
        return $results;
    }

    protected function getTemplateParams()
    {
        $data = array(
            'error_message' => null,
            'items' => $this->cart->getItemsArray(),
            'free_delivery_price' => $this->config->getFreePostagePrice(),
            'tax_rate' => $this->config->getTaxRate(),
            'total_weight' => $this->cart->totalWeight(),
            'weight' => $this->cart->totalWeight(),
        );
        $labels = $this->config->cartItemOptionLabels();
        $data = array_merge($data, $labels);
        $slip_data = $this->slip->getProperties();
        $data = array_merge($data, $slip_data);
        return $data;
    }

    public function logOrder($message)
    {
        if (\App::config()->getLogOrder()) {
            \App::logger()->info($message);
        }
    }

    protected function _createMessage($from, $to, $cc, $bcc, $replyTo, $subject, $body)
    {
        $message = (new \Swift_Message())
                                 ->setMaxLineLength(0)
                                 ->setSubject($subject)
                                 ->setFrom($from)
                                 ->setTo($to)
                                 ->setBody($body);
        if($this->charset === 'iso-2022-jp') {
            $encoder = new \Swift_Mime_ContentEncoder_PlainContentEncoder('7bit');
            $message->setCharset($this->charset)
                ->setEncoder($encoder);
        }
        if (!empty($cc)) {
            $message->setCc($cc);
        }
        if (!empty($bcc)) {
            $message->setBcc($bcc);
        }
        if (!empty($replyTo)) {
            $message->setReplyTo($replyTo);
        }
        return $message;
    }
   
    protected function toAddress($array)
    {
        $addresses = [];
        foreach($array as $address => $name) {
            if(is_numeric($address)) {
                $addresses[] = new Address($name); 
            }
            else {
                $addresses[] = new Address($address, $name); 
            }
        }
        return $addresses;
    }

    protected function createMessage($from, $to, $cc, $bcc, $replyTo, $subject, $body)
    {
        $message = (new Email())
            ->subject($subject)
            ->text($body);
        if(!empty($from)) {
            $message->from(...$this->toAddress($from));
        }
        if(!empty($to)) {
            $message->to(...$this->toAddress($to));
        }
        if(!empty($cc)) {
            $message->cc(...$this->toAddress($cc));
        }
        if(!empty($bcc)) {
            $message->bcc(...$this->toAddress($bcc));
        }
        if(!empty($replyTo)) {
            $message->replyTo(...$this->toAddress($replyTo));
        }
        return $message;
    }

    protected function _sendMessage($message)
    {
        $retry = \App::config()->get('mail_max_retry', 3);
        while($retry--) {
            try {
                // num sent > 0
                if($this->mailer->send($message)) {
                    return;
                }
            }
            catch(\Exception $e) {
                \App::logger()->error("メールが送信できません: " . $e->getMessage());
            }
            $this->mailer->getTransport()->stop();
            sleep(3);
        }

        $error = "メールが送信できません: 再送回数オーバー";
        throw new \RuntimeException($error);
    }

    protected function sendMessage($message)
    {
        $retry = \App::config()->get('mail_max_retry', 3);
        while($retry--) {
            try {
                $this->mailer->send($message);
                return;
            }
            catch(\Exception $e) {
                \App::logger()->error("メールが送信できません: " . $e->getMessage());
            }
            // $this->mailer->getTransport()->stop();
            sleep(3);
        }

        $error = "メールが送信できません: 再送回数オーバー";
        throw new \RuntimeException($error);
    }

    public function sendTestMail($to)
    {
        $settings = $this->config->getMailSettings();
        $from = $settings[0]['from'];
        if(!$from) {
            throw new \RuntimeException('Invalid mail settings - from');
        }
        $from = $this->createMailAddressArray($from);

        if(!$to) {
            throw new \InvalidArgumentException('Missing argument - to');
        }
        if(!filter_var($to, FILTER_VALIDATE_EMAIL)) {
            throw new \InvalidArgumentException('Invalid email address');
        }
        $to = $this->createMailAddressArray($to);

        $subject = 'Test mail from SkeletonCart';
        $body = 'This is a test mail from SkeletonCart.';
        $message = $this->createMessage($from, $to, null, null, null, $subject, $body);
        $this->sendMessage($message);
    }
}

